/*
 * xcorner
 *
 * Turns the display off when you move the mouse pointer to the upper right
 * corner of the screen.
 *
 * Written and copyright by Jim Rees August 2024
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 */

#define _DEFAULT_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Paned.h>

#include "Draw.h"

static void incb(Widget w, XtPointer a1, XtPointer a2);
static void timeproc(XtPointer p, XtIntervalId *idp);

static char appname[] = "Xcorner";

static
String fallback_resources[] = {
    "*geometry:					4x4--2+-2",
    "*foreground:				black",
    "*background:				white",
    "*command:					xset dpms force off",
    "*timeout:					1000",
    NULL
};

typedef struct _AppResources {
    String	command;
    int		timeout;
} AppResources;

AppResources app_resources;

static
XtResource resources[] = {
    {"command", "Command", XtRString, sizeof(String),
     XtOffset(AppResources *, command), XtRImmediate, ""},
    {"timeout", "Timeout", XtRInt, sizeof(int),
     XtOffset(AppResources *, timeout), XtRImmediate, 0},
};

static
XrmOptionDescRec options[] = {
    {"-command",	"command",	XrmoptionSepArg,	NULL},
    {"-timeout",	"timeout",	XrmoptionSepArg,	NULL},
};

int main(int ac, char *av[])
{
    XtAppContext app;
    Display *dpy;
    Window win;
    Widget top, paned, draw;

    top = XtAppInitialize(&app, appname, options, XtNumber(options), &ac, av,
			  fallback_resources, NULL, 0);
    XtGetApplicationResources(top, (XtPointer) &app_resources,
			      resources, XtNumber(resources), NULL, 0);
    dpy = XtDisplay(top);
    paned = XtCreateManagedWidget("paned", panedWidgetClass, top, NULL, 0);
    draw = XtCreateManagedWidget("draw", drawWidgetClass, paned, NULL, 0);

    XtAddCallback(draw, XtNcallback, incb, NULL);

    XtRealizeWidget(top);
    win = XtWindow(draw);
    XClearWindow(dpy, win);
    handle_wm_del(app, top);

    XtAppMainLoop(app);

    XtDestroyWidget(top);
    XtDestroyApplicationContext(app);
    exit(0);
}

static void
incb(Widget w, XtPointer a1, XtPointer a2)
{
    XtAppContext app;
    XEvent *ev = a2;
    static XtIntervalId ilid;

    app = XtWidgetToApplicationContext(w);

    switch(ev->xany.type) {
    case EnterNotify:
	ilid = XtAppAddTimeOut(app, app_resources.timeout, timeproc, app_resources.command);
	break;
    case LeaveNotify:
	if (ilid) {
	    XtRemoveTimeOut(ilid);
	    ilid = 0;
	}
	break;
    case KeyPress:
	if (ev_to_key(ev) == 'a')
	    printf("xcorner by Jim Rees\n");
	else if (ev_to_key(ev) == 'q')
	    XtAppSetExitFlag(app);
	break;
    }
}

static void
timeproc(XtPointer p, XtIntervalId *idp)
{
    system((char *) p);
}
